#!/usr/bin/env python
#
# Time Drive 0.2.0 - based on duplicity - Encrypted bandwidth efficient 
# backup. Requires Duplicity Version 0.6.04 released August 01, 2009
#
# Copyright 2009 Rob Oakes	<LyX-Devel@oak-tree>

import sys, os

from PyQt4 import QtCore, QtGui

from ui_MainWindow import Ui_MainWindow
from settingsdialog import SettingsDialog
from restoredialog import RestoreDialog

import utils
from utils.usernotifications import UserNotifications

from background import testconnectionurl, backupjob, restorefiles
from background import loadsnapshots, managebackups

from backupsettings import BackupSettings
from backupsettings import globals

class MainWindow(QtGui.QWidget):
	"""Main Window"""

	def __init__(self):
		QtGui.QWidget.__init__(self)
		self.ui = Ui_MainWindow()
		self.ui.setupUi(self)
		settings = BackupSettings()
		self.NotifyUser = UserNotifications()
		
		# Create Background Threads
		self.TestConnectionUrl = testconnectionurl.TestConnectionUrl(self)
		self.BackupJob = backupjob.BackupJob(self)
		self.RestoreFiles = restorefiles.RestoreFiles(self)
		self.LoadSnapshots = loadsnapshots.LoadSnapshots(self)
		self.ManageBackups = managebackups.ManageBackups(self)
		
		# Setup SettingsDialog, RestoreDialog
		self.ui.windowRestore = RestoreDialog(self)
		self.ui.windowSettings = SettingsDialog(self, settings)
		self.RemoteResults = None
		
		# Set Up the Restore Files List
		self.modelRestoreList = self.setupRestoreList()
		self.ui.listRestoreFiles.setModel(self.modelRestoreList)
		self.ui.listRestoreFiles.setColumnHidden(2, True)
		self.ui.listRestoreFiles.setColumnHidden(3, True)
		self.ui.listRestoreFiles.setColumnHidden(4, True)
		self.ui.listRestoreFiles.setColumnHidden(5, True)
		self.ui.listRestoreFiles.setColumnHidden(6, True)
		self.ui.listRestoreFiles.setColumnWidth(0, 190)
		self.ui.listRestoreFiles.setColumnWidth(1, 150)
		
		# Set Up the Snapshots List
		self.modelSnapshotsList = self.setupSnapshotsList()
		self.ui.treeSnapshots.setModel(self.modelSnapshotsList)
		self.ui.treeSnapshots.setColumnHidden(1, True)
		self.ui.treeSnapshots.setColumnHidden(2, True)
		self.ui.treeSnapshots.setColumnHidden(3, True)
		self.ui.treeSnapshots.setCurrentIndex(self.modelSnapshotsList.index(0,0))
		self.ui.buttonRefreshSnapshot.setEnabled(False)
		
		# Hide Advanced Restore and Progress Indicators
		self.AdvancedRestore_SetVisible(False)
		self.ProgressIndicators_SetVisible(False)
		self.ui.buttonAdvancedRestore_LoadDirectory.setVisible(False)
		
		# Connect Events and Methods - Toolbar Buttons
		QtCore.QObject.connect(self.ui.buttonSettings, QtCore.SIGNAL("pressed()"), 
			self.buttonSettings_Pressed)
		QtCore.QObject.connect(self.ui.buttonAddToQueue, QtCore.SIGNAL("pressed()"), 
			self.buttonAddToQueue_Pressed)
		QtCore.QObject.connect(self.ui.buttonAdvancedRestore, QtCore.SIGNAL("pressed()"), 
			self.buttonAdvancedRestore_Pressed)
		QtCore.QObject.connect(self.ui.buttonAdvancedRestore_Test, 
			QtCore.SIGNAL("pressed()"), self.TestAdvancedRestoreOptions)
		QtCore.QObject.connect(self.ui.buttonBackup, QtCore.SIGNAL("pressed()"),
			self.buttonBackup_Pressed)
		QtCore.QObject.connect(self.ui.buttonRestoreFiles, QtCore.SIGNAL("clicked()"),
			self.RestoreFiles_Start)
		QtCore.QObject.connect(self.ui.buttonRemoveFromQueue, 
			QtCore.SIGNAL("clicked()"), self.buttonRemoveFromQueue_Pressed)
		QtCore.QObject.connect(self.ui.comboRemoteOptions, 
			QtCore.SIGNAL("activated(const QString &)"),
			self.AdvancedRestore_comboRemoteOptions_currentIndexChanged)
		QtCore.QObject.connect(self.ui.buttonAdvancedRestore_LoadDirectory, 
			QtCore.SIGNAL("pressed()"), self.AdvancedRestore_PickFolder)
		QtCore.QObject.connect(self.ui.buttonRefreshSnapshot, 
			QtCore.SIGNAL("clicked()"), self.buttonRefreshSnapshotList_Pressed)
		QtCore.QObject.connect(self.ui.buttonExit, QtCore.SIGNAL("clicked()"),
			self.buttonExit_Pressed)
			
		# Signals/Slots Related to Background Threads
		QtCore.QObject.connect(self.TestConnectionUrl, QtCore.SIGNAL("finished()"), 
			self.ShowRemoteResults)
		QtCore.QObject.connect(self.TestConnectionUrl, QtCore.SIGNAL("output(bool)"), 
			self.SetRemoteResults)
		
		QtCore.QObject.connect(self.BackupJob, QtCore.SIGNAL("started()"),
			self.backupStarted)
		QtCore.QObject.connect(self.BackupJob, 
			QtCore.SIGNAL("taskComplete(QString, QString)"), self.backupFinished)
		QtCore.QObject.connect(self.BackupJob, 
			QtCore.SIGNAL("taskError(QString, QString)"), self.backupError)
		
		QtCore.QObject.connect(self.ui.windowRestore.LoadArchive, 
			QtCore.SIGNAL("toggleControls(bool)"), self.RestoreDialog_Show)
		QtCore.QObject.connect(self.ui.windowRestore.LoadArchive,
			QtCore.SIGNAL("toggleControls(bool)"), self.ui.buttonRefreshSnapshot.setEnabled)
		QtCore.QObject.connect(self.ui.windowRestore.LoadArchive, 
			QtCore.SIGNAL("setupTaskProgress(int, QString)"), self.ProgressIndicators_Setup)
		QtCore.QObject.connect(self.ui.windowRestore.LoadArchive, 
			QtCore.SIGNAL("updateTaskProgress(int)"), self.ProgressIndicators_Update)
		QtCore.QObject.connect(self.ui.windowRestore.LoadArchive, 
			QtCore.SIGNAL("taskComplete(QString, QString)"), self.ProgressIndicators_Shutdown)
		
		QtCore.QObject.connect(self.RestoreFiles, QtCore.SIGNAL("started()"),
			self.restoreFilesStarted)
		QtCore.QObject.connect(self.RestoreFiles, 
			QtCore.SIGNAL("setupTaskProgress(int, QString)"),
			self.ProgressIndicators_Setup)
		QtCore.QObject.connect(self.RestoreFiles, 
			QtCore.SIGNAL("updateTaskProgress(int)"),
			self.ProgressIndicators_Update)
		QtCore.QObject.connect(self.RestoreFiles,
			QtCore.SIGNAL("taskComplete(QString, QString)"), self.restoreFilesFinished)
		
		QtCore.QObject.connect(self.LoadSnapshots, 
			QtCore.SIGNAL("ReturnSnapshotList_Main(QStandardItemModel*)"), 
			self.LoadSnapshots_SetModelList)
		QtCore.QObject.connect(self.LoadSnapshots, QtCore.SIGNAL("started()"), 
			self.loadSnapshotsStarted)
		QtCore.QObject.connect(self.LoadSnapshots, 
			QtCore.SIGNAL("setupTaskProgress(int, QString)"),
			self.ProgressIndicators_Setup)
		QtCore.QObject.connect(self.LoadSnapshots, QtCore.SIGNAL("finished()"), 
			self.loadSnapshotsFinished)
			
		QtCore.QObject.connect(self.ManageBackups, 
			QtCore.SIGNAL("setupTaskProgress(int, QString)"), 
			self.ProgressIndicators_Setup)
		QtCore.QObject.connect(self.ManageBackups, 
			QtCore.SIGNAL("taskComplete(QString, QString)"),
			self.manageBackups_Finished)
		QtCore.QObject.connect(self.ManageBackups, QtCore.SIGNAL("toggleControls(bool)"),
			self.ui.buttonRefreshSnapshot.setEnabled)
		QtCore.QObject.connect(self.ManageBackups, QtCore.SIGNAL("toggleControls(bool)"),
			self.BackupControls_Toggle)
		QtCore.QObject.connect(self.ManageBackups, QtCore.SIGNAL("toggleControls(bool)"),
			self.RestoreOptions_Toggle)
		
		# Run Startup Events
		self.LoadSnapshots.MainList_FromSettings(settings, False)
	
	
	def setupRestoreList(self, currentModel = None):
		if currentModel == None:
			restoreList = QtGui.QStandardItemModel()
		else:
			restoreList = currentModel
			
		restoreList.setHorizontalHeaderItem(0, QtGui.QStandardItem("File Name"))
		restoreList.setHorizontalHeaderItem(1, QtGui.QStandardItem("Date Modified"))
		restoreList.setHorizontalHeaderItem(2, QtGui.QStandardItem("Relative Path"))
		restoreList.setHorizontalHeaderItem(3, QtGui.QStandardItem("Archive Url"))
		restoreList.setHorizontalHeaderItem(4, QtGui.QStandardItem("Archive Passphrase"))
		restoreList.setHorizontalHeaderItem(5, QtGui.QStandardItem("Restore Time"))
		restoreList.setHorizontalHeaderItem(6, QtGui.QStandardItem("S3 Login"))
		
		return restoreList
	
	
	def setupSnapshotsList(self, currentModel = None):
		if currentModel == None:
			snapshotsList = QtGui.QStandardItemModel()
		else:
			snapshotsList = currentModel
			
		snapshotsList.setHorizontalHeaderItem(0, QtGui.QStandardItem("Snapshot History"))
		snapshotsList.setHorizontalHeaderItem(1, QtGui.QStandardItem("Timestamp"))
		snapshotsList.setHorizontalHeaderItem(2, QtGui.QStandardItem("Int"))
		snapshotsList.setHorizontalHeaderItem(3, QtGui.QStandardItem("Backup Type"))
		itemMostRecent = QtGui.QStandardItem("Most Recent")
		itemTime = QtGui.QStandardItem("")
		
		snapshotsList.appendRow([itemMostRecent, itemTime])
		
		return snapshotsList
	
	
	def buttonSettings_Pressed(self):
		self.ui.windowSettings.show()
	
	
	def buttonAddToQueue_Pressed(self):
		if self.ui.buttonAdvancedRestore.isChecked():
			archiveUrl = self.AdvancedRestore_FetchRemoteUrl()
			gnu_passphrase = self.ui.txtAdvancedRestore_EncryptionPassphrase.text()
			
			try:
				if self.ui.comboRemoteOptions.currentText() == "AmazonS3":
					self.ui.windowRestore.LoadArchive_Remote(archiveUrl,
						gnu_passphrase, True)
				else:
					self.ui.windowRestore.LoadArchive_Remote(archiveUrl, 
						gnu_passphrase, False)
			except:
				msgText = "The archive url that you entered is invalid: \n" + archiveUrl
				msgBoxInvalidUrl = QtGui.QMessageBox.warning(self, 
					"Invalid Archive Url", msgText, QtGui.QMessageBox.Ok)
		else:
			curSelection = self.ui.treeSnapshots.currentIndex()
			restoreTime = self.modelSnapshotsList.data(curSelection.sibling(curSelection.row(), 1)).toString()
			self.ui.windowRestore.timedata = self.CreateTimeData()
			
			if restoreTime != '':
				self.ui.windowRestore.LoadArchive_FromSettings(restoreTime)
			else:
				self.ui.windowRestore.LoadArchive_FromSettings()
	
	
	def buttonRemoveFromQueue_Pressed(self):
		toRemove = self.ui.listRestoreFiles.currentIndex()
		if toRemove.isValid():
			self.modelRestoreList.removeRow(toRemove.row(),toRemove.parent())
	
	def loadSnapshotsStarted(self):
		self.RestoreOptions_Toggle(False)

	def loadSnapshotsFinished(self):
		self.RestoreOptions_Toggle(True)
		self.ui.buttonRefreshSnapshot.setEnabled(True)
		self.ProgressIndicators_Shutdown()

	def restoreFilesStarted(self):
		# Disable the Restore Options
		self.RestoreOptions_Toggle(False)
		
		self.doNotify("Restoring Files from Archive")
	
	def restoreFilesFinished(self, taskName, message):
		self.RestoreOptions_Toggle(True)
		self.RestoreFiles_RemoveFromList(True)
		self.ProgressIndicators_Shutdown()
		self.doNotify(message)


	def RestoreOptions_Toggle(self, option):
		self.ui.buttonAddToQueue.setEnabled(option)
		self.ui.buttonRemoveFromQueue.setEnabled(option)
		self.ui.buttonFileInformation.setEnabled(option)
		self.ui.buttonAdvancedRestore.setEnabled(option)
		self.ui.buttonRestoreFiles.setEnabled(option)
	
	
	def RestoreDialog_Show(self, option):
		"""Show the restore files dialog after a file list has been loaded."""
		self.RestoreOptions_Toggle(option)
		if option == True:
			self.ui.windowRestore.show()
		
	
	def buttonAdvancedRestore_Pressed(self):
		self.AdvancedRestore_SetVisible(not self.ui.buttonAdvancedRestore.isChecked())
	
	
	def AdvancedRestore_FetchRemoteUrl(self):
		"""Constructs a valid remote url from the Advanced Restore Dialog"""
		
		connectionType = self.ui.comboRemoteOptions.currentText()
		connectionPath = self.ui.txtAdvancedRestore_Url.text()
		connectionUsername = self.ui.txtAdvancedRestore_Username.text()
		connectionPassword = self.ui.txtAdvancedRestore_Password.text()
		if connectionType == "file":
			completeUrl = connectionType + '://' + connectionPath
		elif connectionType == "AmazonS3":
			completeUrl = "s3+http://" + os.path.normpath(str(connectionPath)) + " " + connectionUsername + " " + connectionPassword
		else:
			completeUrl = connectionType + '://' + connectionUsername + ':' + connectionPassword + '@' + connectionPath
		return completeUrl
	
	
	def AdvancedRestore_SetVisible(self, status):
		"""Toggles the Advanced Restore Pane Visbile/Invisible"""
		self.ui.buttonAdvancedRestore_Test.setVisible(status)
		self.ui.buttonHelp_2.setVisible(status)
		self.ui.comboRemoteOptions.setVisible(status)
		self.ui.lblAdvancedRestore_Url.setVisible(status)
		self.ui.txtAdvancedRestore_Url.setVisible(status)
		self.ui.lblAdvancedRestore_Password.setVisible(status)
		self.ui.txtAdvancedRestore_Password.setVisible(status)
		self.ui.txtAdvancedRestore_Username.setVisible(status)
		self.ui.lblAdvancedRestore_Username.setVisible(status)
		self.ui.lblAdvancedRestore_EncryptionPassphrase.setVisible(status)
		self.ui.txtAdvancedRestore_EncryptionPassphrase.setVisible(status)
		self.ui.lblTstResult.setVisible(False)
		self.ui.treeSnapshots.setCurrentIndex(self.modelSnapshotsList.index(0,0))
		self.ui.treeSnapshots.setEnabled(not status)
		
		if (status == True) & (self.ui.comboRemoteOptions.currentIndex() == 1):
			self.ui.buttonAdvancedRestore_LoadDirectory.setVisible(True)
		else:
			self.ui.buttonAdvancedRestore_LoadDirectory.setVisible(False)
	
	
	def AdvancedRestore_comboRemoteOptions_currentIndexChanged(self, index):
		"""Toggles the file selection button on/off depending on the
		remote option selected."""
		
		def setDefaultLabels():
			self.ui.lblAdvancedRestore_Username.setText("Username")
			self.ui.lblAdvancedRestore_Password.setText("Password")
		
		def setLoginEnabled(option):
			self.ui.lblAdvancedRestore_Username.setEnabled(option)
			self.ui.lblAdvancedRestore_Password.setEnabled(option)
			self.ui.txtAdvancedRestore_Username.setEnabled(option)
			self.ui.txtAdvancedRestore_Password.setEnabled(option)
		
		if index == "file":
			self.ui.buttonAdvancedRestore_LoadDirectory.setVisible(True)
			setDefaultLabels()
			setLoginEnabled(False)
		elif index == "AmazonS3":
			self.ui.buttonAdvancedRestore_LoadDirectory.setVisible(False)
			setLoginEnabled(True)
			self.ui.lblAdvancedRestore_Username.setText("Access ID")
			self.ui.lblAdvancedRestore_Password.setText("Secret Key")
		else:
			self.ui.buttonAdvancedRestore_LoadDirectory.setVisible(False)
			setDefaultLabels()
			setLoginEnabled(True)
	
	
	def AdvancedRestore_PickFolder(self):
		folderName = QtGui.QFileDialog.getExistingDirectory()
		
		if folderName != '':
			self.ui.txtAdvancedRestore_Url.setText(folderName)
	
	def ProgressIndicators_SetVisible(self, status):
		self.ui.statusLabel.setVisible(status)
		self.ui.backupStatus.setVisible(status)
		
	
	def BackupControls_Toggle(self, option):
		self.ui.buttonBackup.setEnabled(option)
		self.ui.buttonRefreshSnapshot.setEnabled(option)
		self.ui.buttonNameSnapshot.setEnabled(option)

	
	def TestAdvancedRestoreOptions(self):
		"""Test the connection to see if it is valid"""
		archiveUrl = self.AdvancedRestore_FetchRemoteUrl()
		self.ui.buttonAdvancedRestore_Test.setEnabled(False)
		if self.ui.comboRemoteOptions.currentText() == "AmazonS3":
			archiveUrl, S3_AccessId, S3_SecretKey = str(archiveUrl).split()
			S3_LoginInfo = S3_AccessId + " " + S3_SecretKey
			self.TestConnectionUrl.TestUrl(archiveUrl, S3_LoginInfo)
		else:
			self.TestConnectionUrl.TestUrl(archiveUrl)
	
	
	def buttonBackup_Pressed(self):
		settings = BackupSettings()
		self.BackupJob.StartBackup(settings)

	
	def backupStarted(self):
		"""Disable Backup Related Ui Controls and notify user that the backup
		has started (Ubuntu 9.04, Mac OS X)"""
		
		self.BackupControls_Toggle(False)
		self.doNotify("Starting File Backup")
	
	
	def backupFinished(self, taskName, message):
		"""Notify User that the Backup has ended
		(Ubuntu 9.04, Mac OS X)"""
		
		self.BackupControls_Toggle(True)
		self.RefreshSnapshotList(True)
		self.doNotify(message)
	
	
	def backupError(self, taskName, message):
		"""Notify user that there was an error."""
		self.doNotify(message)
	
	
	def manageBackups_Finished(self, taskName, message):
		"""Notify user that the operation has completed successfully.
		Refresh the snapshot list."""
		self.RefreshSnapshotList(True)
		self.doNotify(message)
	
	
	def SetRemoteResults(self, tstResult):
		self.RemoteResults = tstResult
	
	
	def ShowRemoteResults(self):
		"""Updates the settings view based on the results of the connection test."""
		self.ui.lblTstResult.setVisible(True)
		
		if self.RemoteResults == True:
			imageSuccess = QtGui.QPixmap(':/PNG/Resources/Dialog-Ok.png')
			self.ui.lblTstResult.setPixmap(imageSuccess)
		if self.RemoteResults == False:
			imageFailure = QtGui.QPixmap(':/PNG/Resources/Dialog-Cancel.png')
			self.ui.lblTstResult.setPixmap(imageFailure)
		
		self.ui.buttonAdvancedRestore_Test.setEnabled(True)
		self.RemoteResults = None
	
	
	def RestoreFiles_Start(self):
		settings = BackupSettings()
		RestoreFiles = { }
		i = 0
		
		while i < self.modelRestoreList.rowCount():
			restoreKey = QtCore.QPersistentModelIndex(self.modelRestoreList.index(i,0))
			RelPath = self.modelRestoreList.data(restoreKey.sibling(restoreKey.row(),2)).toString()
			ArchiveUrl = self.modelRestoreList.data(restoreKey.sibling(restoreKey.row(),3)).toString()
			PassPhrase = str(self.modelRestoreList.data(restoreKey.sibling(restoreKey.row(),4)).toString())
			RestoreTime = self.modelRestoreList.data(restoreKey.sibling(restoreKey.row(), 5)).toString()
			S3_LoginInfo = str(self.modelRestoreList.data(restoreKey.sibling(restoreKey.row(), 6)).toString())
			
			RestoreFiles[restoreKey] = [RelPath, ArchiveUrl, PassPhrase, 
				RestoreTime, S3_LoginInfo]
			i = i+1

		if settings.Advanced_DefaultRestoreEnabled == True:
			self.RestoreFiles.StartRestore(RestoreFiles, settings)
		else:
			self.RestoreFiles.StartRestore(RestoreFiles, settings,QtGui.QFileDialog.getExistingDirectory())
	
	
	def RestoreFiles_RemoveFromList(self, operationComplete):
		"""Clears the restore queue of all files."""
		if operationComplete:
			self.modelRestoreList.clear()
			self.setupRestoreList(self.modelRestoreList)
			self.ui.listRestoreFiles.setColumnHidden(2, True)
			self.ui.listRestoreFiles.setColumnHidden(3, True)
			self.ui.listRestoreFiles.setColumnHidden(4, True)
			self.ui.listRestoreFiles.setColumnHidden(5, True)
			self.ui.listRestoreFiles.setColumnHidden(6, True)
			self.ui.listRestoreFiles.setColumnWidth(0, 190)
			self.ui.listRestoreFiles.setColumnWidth(1, 150)
	
	
	def ProgressIndicators_Setup(self, maximum, message = None):
		self.ProgressIndicators_SetVisible(True)
		self.ui.backupStatus.setMinimum(0)
		self.ui.backupStatus.setMaximum(maximum)
		if message != None:
			self.ui.statusLabel.setText(QtCore.QString(message))
	
	
	def ProgressIndicators_Update(self, progress, message = None):
		self.ui.backupStatus.setValue(progress)
		if message != None:
			self.ui.statusLabel.setText(message)
		
	
	def ProgressIndicators_Shutdown(self):
		self.ProgressIndicators_SetVisible(False)
		self.ui.statusLabel.setText("")
		self.ui.backupStatus.setMinimum(0)
		self.ui.backupStatus.setMaximum(0)
	
	
	def buttonRefreshSnapshotList_Pressed(self):
		self.RefreshSnapshotList(True)
	
	
	def RefreshSnapshotList(self, sync = False):
		settings = BackupSettings()
		self.ui.buttonRefreshSnapshot.setEnabled(False)
		
		# Clear the Existing Snapshots, Reset Column Headers
		self.modelSnapshotsList.clear()
		self.modelSnapshotsList.setHorizontalHeaderItem(0, QtGui.QStandardItem('Snapshot History'))
		self.modelSnapshotsList.setHorizontalHeaderItem(1, QtGui.QStandardItem('Timestamp'))
		self.ui.treeSnapshots.setColumnHidden(1, True)
		
		# Set First Item to "Most Recent"
		itemMostRecent = QtGui.QStandardItem("Most Recent")
		itemTime = QtGui.QStandardItem("")
		
		self.modelSnapshotsList.appendRow([itemMostRecent, itemTime])
		self.LoadSnapshots.MainList_FromSettings(settings, sync)
		self.ui.treeSnapshots.setCurrentIndex(itemMostRecent.index())
	
	
	def LoadSnapshots_SetModelList(self, transferModel):
		"""Loads the available snapshots to treeSnapshots"""
		receiveModel = self.modelSnapshotsList
		i = 0
		
		iconFullBackup = QtGui.QIcon(QtGui.QPixmap(":/PNG/Resources/Full-Backup-Emblem.png"))
		iconIncBackup = QtGui.QIcon(QtGui.QPixmap(":/PNG/Resources/Inc-Backup-Emblem.png"))
		numRows = transferModel.rowCount()
		
		while i < numRows:
			moveRow = transferModel.takeRow(0)
			receiveModel.appendRow(moveRow)
			backupType = receiveModel.data(receiveModel.index(i+1,3)).toString()
			itemTimePretty = receiveModel.itemFromIndex(receiveModel.index(i+1,0))
			
			if backupType == "full":
				itemTimePretty.setIcon(iconFullBackup)
			else:
				itemTimePretty.setIcon(iconIncBackup)
			
			i = i+1
		
		self.ui.treeSnapshots.setColumnHidden(1, True)
		self.ui.treeSnapshots.setColumnHidden(2, True)
		self.ui.treeSnapshots.setColumnHidden(3, True)
	
	
	def buttonExit_Pressed(self):
		self.close()
	
	
	def CreateTimeData(self):
		timeData = { }
		i = 1
		
		while i < self.modelSnapshotsList.rowCount():
			restoreKey = self.modelSnapshotsList.index(i,0)
			timeInt = self.modelSnapshotsList.data(restoreKey.sibling(restoreKey.row(),2)).toInt()
			timeString = self.modelSnapshotsList.data(restoreKey.sibling(restoreKey.row(), 1)).toString()
			timePretty = self.modelSnapshotsList.data(restoreKey.sibling(restoreKey.row(), 0)).toString()
			timeData[timeInt[0]] = [timeString, timePretty]
			i = i+1
		
		if self.modelSnapshotsList.rowCount() == 1:
			timeString = utils.time_GetCurrentTime()
			timeInt = utils.time_String2Int(timeString)
			timePretty = utils.time_String2Pretty(timeString)
			timeData[timeInt] = [timeString, timePretty]
			
		return timeData
	
	
	def doNotify(self, userMessage):
		settings = BackupSettings()
		if settings.Options_EnableNotifications == True:
			self.NotifyUser.Notify(globals.APPLICATIONNAME, str(userMessage))
