from PyQt4 import QtCore
import backupfolder
import excludeitem
import globals

def readSettings(backupSettings):
	settings = backupSettings.getSettings()
	backupSettings.LocalBackup = settings.value(globals.keyLocalBackup).toBool()
	backupSettings.LocalBackupDirectory = settings.value(globals.keyLocalBackupDirectory).toString()
	
	# Remote Backup Settings
	backupSettings.RemoteBackup = settings.value(globals.keyRemoteBackup).toBool()
	backupSettings.RemoteProtocolInt = settings.value(globals.keyRemoteProtocolInt).toInt()[0]
	backupSettings.RemoteProtocol = settings.value(globals.keyRemoteProtocol).toString()
	backupSettings.RemotePath = settings.value(globals.keyRemotePath).toString()
	backupSettings.RemoteUsername = settings.value(globals.keyUsername).toString()
	backupSettings.RemotePassword = decode(settings.value(globals.keyPassword).toString())

	#Amazon S3 Backup Settings
	backupSettings.AmazonS3Backup = settings.value(globals.keyAmazonS3Backup).toBool()
	backupSettings.AmazonS3_Bucket = settings.value(globals.keyAmazonS3Bucket).toString()
	backupSettings.AmazonS3_AccessId = settings.value(globals.keyAmazonS3AccessId).toString()
	backupSettings.AmazonS3_SecretKey = decode(settings.value(globals.keyAmazonS3SecretKey).toString())
	backupSettings.AmazonS3_LocationInt = settings.value(globals.keyAmazonS3LocationInt).toInt()[0]
	backupSettings.AmazonS3_Location = settings.value(globals.keyAmazonS3Location).toString()

	# Folder Include Settings
	numFolders = settings.beginReadArray(globals.keyIncludeFolders)
	for i in range(numFolders):
		settings.setArrayIndex(i)
		f = backupfolder.BackupFolder()
		f.FolderName = settings.value(globals.keyArrayFolderName).toString()
		f.BackupType = settings.value(globals.keyArrayBackupType).toInt()[0]
		f.LocalBackupDirectory = settings.value(globals.keyArrayLocalBackupDir).toString()
		f.RemoteProtocolInt = settings.value(globals.keyArrayRemoteProtocolInt).toInt()[0]
		f.RemoteProtocol = settings.value(globals.keyArrayRemoteProtocol).toString()
		f.Username = settings.value(globals.keyArrayUsername).toString()
		f.Password = decode(settings.value(globals.keyArrayPassword).toString())
		f.RemotePath = settings.value(globals.keyArrayRemotePath).toString()
		f.EncryptionKeyEnabled = settings.value(globals.keyArrayEncryptionKeyEnabled).toBool()
		f.EncryptionKey = settings.value(globals.keyArrayEncryptionKey).toString()
		
		# Read the Exclude Patterns, Files and Folders
		numExcludeItems = settings.beginReadArray(globals.keyFolderExclusions)
		for k in range(numExcludeItems):
			settings.setArrayIndex(k)
			e = excludeitem.ExcludeItem(settings.value(globals.keyArrayExclusion).toString())
			e.Type = settings.value(globals.keyArrayExclusionType).toInt()[0]
			f.ExcludeList.append(e)

		settings.endArray()
		backupSettings.FolderList.append(f)			
	
	settings.endArray()

	# Exclude Settings
	numGlobalExclusions = settings.beginReadArray(globals.keyGlobalExclusions)
	for  i in range(numGlobalExclusions):
		settings.setArrayIndex(i)
		e = excludeitem.ExcludeItem(settings.value(globals.keyArrayExclusion).toString())
		e.Type = settings.value(globals.keyArrayExclusionType).toInt()[0]
		backupSettings.ExcludeList.append(e)
	settings.endArray()
	
	# Autoremove Settings - Older Than
	backupSettings.AutoRemove_OlderThan = settings.value(globals.keyAutoRemoveOlderThan).toBool()
	backupSettings.AutoRemove_OlderThan_Duration = settings.value(globals.keyAutoRemoveOlderThanDuration).toInt()[0]
	backupSettings.AutoRemove_OlderThan_Units = settings.value(globals.keyAutoRemoveOlderThanUnits).toInt()[0]
	
	# Autoremove Settings - Free Space
	backupSettings.AutoRemove_FreeSpace = settings.value(globals.keyAutoRemoveFreeSpace).toBool()
	backupSettings.AutoRemove_FreeSpace_Size = settings.value(globals.keyAutoRemoveFreeSpaceSize).toInt()[0]
	backupSettings.AutoRemove_FreeSpace_Units = settings.value(globals.keyAutoRemoveFreeSpaceUnits).toInt()[0]
	
	# Autoremove Settings - Smart Remove
	backupSettings.AutoRemove_SmartRemove = settings.value(globals.keyAutoRemoveSmartRemove).toBool()
	backupSettings.AutoRemove_NamedSnapshots = settings.value(globals.keyAutoRemoveNamedSnapshots).toBool()
	
	# Backup Schedule
	backupSettings.Schedule = settings.value(globals.keySchedule).toInt()[0]
	
	# Archive Options
	backupSettings.Options_CompressFiles = settings.value(globals.keyCompressFiles).toBool()
	backupSettings.Options_EncryptFilesEnabled = settings.value(globals.keyEncryptFiles).toBool()
	backupSettings.Options_EncryptionKey = settings.value(globals.keyEncryptionKey).toString()
	backupSettings.Options_EnableNotifications = settings.value(globals.keyEnableNotifications).toBool()
	backupSettings.Options_ExcludeHidden = settings.value(globals.keyExcludeHidden).toBool()
	backupSettings.Options_FullBackupFrequencyEnabled = settings.value(globals.keyFullBackupFrequencyEnabled).toBool()
	backupSettings.Options_FullBackupFrequency = settings.value(globals.keyFullBackupFrequency).toInt()[0]
	
	#advanced Options
	backupSettings.Advanced_DefaultRestoreEnabled = settings.value(globals.keyAdvancedDefaultRestore).toBool()
	backupSettings.Advanced_DefaultRestoreLocation = settings.value(globals.keyAdvancedDefaultRestoreLocation).toString()

def writeSettings(backupSettings):
	settings = backupSettings.getSettings()
	settings.setValue(globals.keyLocalBackup, QtCore.QVariant(backupSettings.LocalBackup))
	settings.setValue(globals.keyLocalBackupDirectory, QtCore.QVariant(str(backupSettings.LocalBackupDirectory)))

	# Remote Backup Settings
	settings.setValue(globals.keyRemoteBackup, QtCore.QVariant(backupSettings.RemoteBackup))
	settings.setValue(globals.keyRemoteProtocolInt, QtCore.QVariant(str(backupSettings.RemoteProtocolInt)))
	settings.setValue(globals.keyRemoteProtocol, QtCore.QVariant(str(backupSettings.RemoteProtocol)))
	settings.setValue(globals.keyRemotePath, QtCore.QVariant(str(backupSettings.RemotePath)))
	settings.setValue(globals.keyUsername, QtCore.QVariant(str(backupSettings.RemoteUsername)))
	settings.setValue(globals.keyPassword, QtCore.QVariant(encode(backupSettings.RemotePassword)))
	
	#Amazon S3 Backup Settings
	settings.setValue(globals.keyAmazonS3Backup, QtCore.QVariant(backupSettings.AmazonS3Backup))
	settings.setValue(globals.keyAmazonS3Bucket, QtCore.QVariant(str(backupSettings.AmazonS3_Bucket)))
	settings.setValue(globals.keyAmazonS3AccessId, QtCore.QVariant(str(backupSettings.AmazonS3_AccessId)))
	settings.setValue(globals.keyAmazonS3SecretKey, QtCore.QVariant(encode(backupSettings.AmazonS3_SecretKey)))
	settings.setValue(globals.keyAmazonS3LocationInt, QtCore.QVariant(str(backupSettings.AmazonS3_LocationInt)))
	settings.setValue(globals.keyAmazonS3Location, QtCore.QVariant(str(backupSettings.AmazonS3_Location)))

	# Folders Included in Backup
	settings.remove(globals.keyIncludeFolders)
	settings.beginWriteArray(globals.keyIncludeFolders)
	for f in backupSettings.FolderList:
		settings.setArrayIndex(backupSettings.FolderList.index(f))
		settings.setValue(globals.keyArrayFolderName, QtCore.QVariant(str(f.FolderName)))
		settings.setValue(globals.keyArrayBackupType, QtCore.QVariant(str(f.BackupType)))
		settings.setValue(globals.keyArrayLocalBackupDir, QtCore.QVariant(str(f.LocalBackupDirectory)))
		settings.setValue(globals.keyArrayRemoteProtocolInt, QtCore.QVariant(str(f.RemoteProtocolInt)))
		settings.setValue(globals.keyArrayRemoteProtocol, QtCore.QVariant(str(f.RemoteProtocol)))
		settings.setValue(globals.keyArrayUsername, QtCore.QVariant(str(f.Username)))
		settings.setValue(globals.keyArrayPassword, QtCore.QVariant(encode(f.Password)))
		settings.setValue(globals.keyArrayRemotePath, QtCore.QVariant(str(f.RemotePath)))
		settings.setValue(globals.keyArrayEncryptionKeyEnabled, QtCore.QVariant(f.EncryptionKeyEnabled))
		settings.setValue(globals.keyArrayEncryptionKey, QtCore.QVariant(str(f.EncryptionKey)))

		# Folder Specific Exclusions
		settings.beginWriteArray(globals.keyFolderExclusions)
		numExcludeItems = len(f.ExcludeList)
		for e in f.ExcludeList:
			settings.setArrayIndex(f.ExcludeList.index(e))
			settings.setValue(globals.keyArrayExclusion, QtCore.QVariant(str(e.Item)))
			settings.setValue(globals.keyArrayExclusionType, QtCore.QVariant(str(e.Type)))
		settings.endArray()
		
	settings.endArray()

	# Global Exclude Patterns
	settings.remove(globals.keyGlobalExclusions)
	settings.beginWriteArray(globals.keyGlobalExclusions)
	for e in backupSettings.ExcludeList:
		settings.setArrayIndex(backupSettings.ExcludeList.index(e))
		settings.setValue(globals.keyArrayExclusion, QtCore.QVariant(str(e.Item)))
		settings.setValue(globals.keyArrayExclusionType, QtCore.QVariant(str(e.Type)))
	settings.endArray()
	
	# Autoremove Settings - Older Than
	settings.setValue(globals.keyAutoRemoveOlderThan, QtCore.QVariant(backupSettings.AutoRemove_OlderThan))
	settings.setValue(globals.keyAutoRemoveOlderThanDuration, QtCore.QVariant(str(backupSettings.AutoRemove_OlderThan_Duration)))
	settings.setValue(globals.keyAutoRemoveOlderThanUnits, QtCore.QVariant(str(backupSettings.AutoRemove_OlderThan_Units)))
	
	# Autoremove Settings - Free Space
	settings.setValue(globals.keyAutoRemoveFreeSpace, QtCore.QVariant(backupSettings.AutoRemove_FreeSpace))
	settings.setValue(globals.keyAutoRemoveFreeSpaceSize, QtCore.QVariant(str(backupSettings.AutoRemove_FreeSpace_Size)))
	settings.setValue(globals.keyAutoRemoveFreeSpaceUnits, QtCore.QVariant(str(backupSettings.AutoRemove_FreeSpace_Units)))

	# Autoremove Settings - Smart Remove
	settings.setValue(globals.keyAutoRemoveSmartRemove, QtCore.QVariant(backupSettings.AutoRemove_SmartRemove))
	settings.setValue(globals.keyAutoRemoveNamedSnapshots, QtCore.QVariant(backupSettings.AutoRemove_NamedSnapshots))
	
	# Backup Schedule
	settings.setValue(globals.keySchedule, QtCore.QVariant(backupSettings.Schedule))
	
	# Archive Options
	settings.setValue(globals.keyCompressFiles, QtCore.QVariant(backupSettings.Options_CompressFiles))
	settings.setValue(globals.keyEncryptFiles, QtCore.QVariant(backupSettings.Options_EncryptFilesEnabled))
	settings.setValue(globals.keyEncryptionKey, QtCore.QVariant(str(backupSettings.Options_EncryptionKey)))
	settings.setValue(globals.keyEnableNotifications, QtCore.QVariant(backupSettings.Options_EnableNotifications))
	settings.setValue(globals.keyExcludeHidden, QtCore.QVariant(backupSettings.Options_ExcludeHidden))
	settings.setValue(globals.keyFullBackupFrequencyEnabled, QtCore.QVariant(backupSettings.Options_FullBackupFrequencyEnabled))
	settings.setValue(globals.keyFullBackupFrequency, QtCore.QVariant(str(backupSettings.Options_FullBackupFrequency)))

	# advanced
	settings.setValue(globals.keyAdvancedDefaultRestore,QtCore.QVariant(backupSettings.Advanced_DefaultRestoreEnabled))
	settings.setValue(globals.keyAdvancedDefaultRestoreLocation, QtCore.QVariant(str(backupSettings.Advanced_DefaultRestoreLocation)))

def encode(password):
	return (str(password).encode("utf8")).encode("base64")

def decode(password):
	return str(password).decode("base64")

class BackupSettings():
	
	_shared_state = {}
	_Settings = None

	_LocalBackup = None
	_LocalBackupDirectory = None
	_RemoteBackup = None
	_RemoteProtocolInt = None
	_RemoteProtocol = None
	_RemotePath = None
	_RemoteUsername = None
	_RemotePassword = None
	_AmazonS3Backup = None
	_AmazonS3_Bucket = None
	_AmazonS3_AccessId = None
	_AmazonS3_SecretKey = None
	_AmazonS3_Location = None
	_AmazonS3_LocationInt = None
	_AutoRemove_OlderThan = None
	_AutoRemove_OlderThan_Duration = None
	_AutoRemove_OlderThan_Units = None
	_AutoRemove_FreeSpace = None
	_AutoRemove_FreeSpace_Size = None
	_AutoRemove_FreeSpace_Units = None	
	_AutoRemove_SmartRemove = None
	_AutoRemove_NamedSnapshots = None	
	_Schedule = None
	_Options_CompressFiles = None
	_Options_EncryptFilesEnabled = None
	_Options_EncryptionKey = None
	_Options_EnableNotifications = None
	_Options_ExcludeHidden = None
    	_Options_FullBackupFrequencyEnabled = None
    	_Options_FullBackupFrequency = None
	_Advanced_DefaultRestoreEnabled = None
	_Advanced_DefaultRestoreLocation = None

	_FolderList = list()
	_ExcludeList = list()

    	def __init__(self):
        	self.__dict__ = self._shared_state
		if self._Settings is None:
			readSettings(self)
	
	def getSettings(self):
		if self._Settings is None:
			self._Settings = QtCore.QSettings()
		return self._Settings

	def getLocalBackup(self):
		return self._LocalBackup

	def setLocalBackup(self, value):
		self._LocalBackup = value

	def getLocalBackupDirectory(self):
		return self._LocalBackupDirectory

	def setLocalBackupDirectory(self, value):
		self._LocalBackupDirectory = value

	def getRemoteBackup(self):
		return self._RemoteBackup

	def setRemoteBackup(self, value):
		self._RemoteBackup = value

	def getRemoteProtocolInt(self):
		return self._RemoteProtocolInt

	def setRemoteProtocolInt(self, value):
		self._RemoteProtocolInt = value

	def getRemoteProtocol(self):
		return self._RemoteProtocol

	def setRemoteProtocol(self, value):
		self._RemoteProtocol = value

	def getRemotePath(self):
		return self._RemotePath

	def setRemotePath(self, value):
		self._RemotePath = value

	def getRemoteUsername(self):
		return self._RemoteUsername

	def setRemoteUsername(self, value):
		self._RemoteUsername = value

	def getRemotePassword(self):
		return self._RemotePassword

	def setRemotePassword(self, value):
		self._RemotePassword = value

	def getAmazonS3Backup(self):
		return self._AmazonS3Backup

	def setAmazonS3Backup(self, value):
		self._AmazonS3Backup = value
  
	def getAmazonS3_Bucket(self):
		return self._AmazonS3_Bucket

	def setAmazonS3_Bucket(self, value):
		self._AmazonS3_Bucket = value

	def getAmazonS3_AccessId(self):
		return self._AmazonS3_AccessId

	def setAmazonS3_AccessId(self, value):
		self._AmazonS3_AccessId = value

	def getAmazonS3_SecretKey(self):
		return self._AmazonS3_SecretKey

	def setAmazonS3_SecretKey(self, value):
		self._AmazonS3_SecretKey = value

	def getAmazonS3_LocationInt(self):
		return self._AmazonS3_LocationInt

	def setAmazonS3_LocationInt(self, value):
		self._AmazonS3_LocationInt = value

	def getAmazonS3_Location(self):
		return self._AmazonS3_Location

	def setAmazonS3_Location(self, value):
		self._AmazonS3_Location = value

	def getAutoRemove_OlderThan(self):
		return self._AutoRemove_OlderThan

	def setAutoRemove_OlderThan(self, value):
		self._AutoRemove_OlderThan = value

	def getAutoRemove_OlderThan_Duration(self):
		return self._AutoRemove_OlderThan_Duration

	def setAutoRemove_OlderThan_Duration(self, value):
		self._AutoRemove_OlderThan_Duration = value

	def getAutoRemove_OlderThan_Units(self):
		return self._AutoRemove_OlderThan_Units

	def setAutoRemove_OlderThan_Units(self, value):
		self._AutoRemove_OlderThan_Units = value

	def getAutoRemove_FreeSpace(self):
		return self._AutoRemove_FreeSpace

	def setAutoRemove_FreeSpace(self, value):
		self._AutoRemove_FreeSpace = value

	def getAutoRemove_FreeSpace_Size(self):
		return self._AutoRemove_FreeSpace_Size

	def setAutoRemove_FreeSpace_Size(self, value):
		self._AutoRemove_FreeSpace_Size = value

	def getAutoRemove_FreeSpace_Units(self):
		return self._AutoRemove_FreeSpace_Units

	def setAutoRemove_FreeSpace_Units(self, value):
		self._AutoRemove_FreeSpace_Units = value

	def getAutoRemove_SmartRemove(self):
		return self._AutoRemove_SmartRemove

	def setAutoRemove_SmartRemove(self, value):
		self._AutoRemove_SmartRemove = value

	def getAutoRemove_NamedSnapshots(self):
		return self._AutoRemove_NamedSnapshots

	def setAutoRemove_NamedSnapshots(self, value):
		self._AutoRemove_NamedSnapshots = value

	def getSchedule(self):
		return self._Schedule
	
	def setSchedule(self, value):
		self._Schedule = value

	def getOptions_CompressFiles(self):
		return self._Options_CompressFiles
	
	def setOptions_CompressFiles(self, value):
		self._Options_CompressFiles = value

	def getOptions_EncryptFilesEnabled(self):
		return self._Options_EncryptFilesEnabled

	def setOptions_EncryptFilesEnabled(self, value):
		self._Options_EncryptFilesEnabled = value

	def getOptions_EncryptionKey(self):
		return self._Options_EncryptionKey
	
	def setOptions_EncryptionKey(self, value):
		self._Options_EncryptionKey = value

	def getOptions_EnableNotifications(self):
		return self._Options_EnableNotifications

	def setOptions_EnableNotifications(self, value):
		self._Options_EnableNotifications = value

	def getOptions_ExcludeHidden(self):
		return self._Options_ExcludeHidden

	def setOptions_ExcludeHidden(self, value):
		self._Options_ExcludeHidden = value

    	def getOptions_FullBackupFrequencyEnabled(self, value):
        	return self._Options_FullBackupFrequencyEnabled

    	def setOptions_FullBackupFrequencyEnabled(self, value):
        	self._Options_FullBackupFrequencyEnabled = value

	def getOptions_FullBackupFrequency(self, value):
        	return self._Options_FullBackupFrequency

    	def setOptions_FullBackupFrequency(self, value):
        	self._Options_FullBackupFrequency = value

	def getAdvanced_DefaultRestoreEnabled(self):
		return self._Advanced_DefaultRestoreEnabled

	def setAdvanced_DefaultRestoreEnabled(self, value):
		self._Advanced_DefaultRestoreEnabled = value

	def getAdvanced_DefaultRestoreLocation(self):
		return self._Advanced_DefaultRestoreLocation

	def setAdvanced_DefaultRestoreLocation(self, value):
		self._Advanced_DefaultRestoreLocation = value

	def getFolderList(self):
		return self._FolderList

	def getExcludeList(self):
		return self._ExcludeList

	Settings = property(getSettings)
	LocalBackup = property(getLocalBackup, setLocalBackup)
	LocalBackupDirectory = property(getLocalBackupDirectory, setLocalBackupDirectory)
	RemoteBackup = property(getRemoteBackup, setRemoteBackup)
	RemoteProtocolInt = property(getRemoteProtocolInt, setRemoteProtocolInt)
	RemoteProtocol = property(getRemoteProtocol, setRemoteProtocol)
	RemotePath = property(getRemotePath, setRemotePath)
	RemoteUsername = property(getRemoteUsername, setRemoteUsername)
	RemotePassword = property(getRemotePassword, setRemotePassword)
	AmazonS3Backup = property(getAmazonS3Backup, setAmazonS3Backup)
	AmazonS3_Bucket = property(getAmazonS3_Bucket, setAmazonS3_Bucket)
	AmazonS3_AccessId = property(getAmazonS3_AccessId, setAmazonS3_AccessId)
	AmazonS3_SecretKey = property(getAmazonS3_SecretKey, setAmazonS3_SecretKey)
	AmazonS3_LocationInt = property(getAmazonS3_LocationInt, setAmazonS3_LocationInt)
	AmazonS3_Location = property(getAmazonS3_Location, setAmazonS3_Location)
	AutoRemove_OlderThan = property(getAutoRemove_OlderThan, setAutoRemove_OlderThan)
	AutoRemove_OlderThan_Duration = property(getAutoRemove_OlderThan_Duration, setAutoRemove_OlderThan_Duration)
	AutoRemove_OlderThan_Units = property(getAutoRemove_OlderThan_Units, setAutoRemove_OlderThan_Units)
	AutoRemove_FreeSpace = property(getAutoRemove_FreeSpace, setAutoRemove_FreeSpace)
	AutoRemove_FreeSpace_Size = property(getAutoRemove_FreeSpace_Size, setAutoRemove_FreeSpace_Size)
	AutoRemove_FreeSpace_Units = property(getAutoRemove_FreeSpace_Units, setAutoRemove_FreeSpace_Units)
	AutoRemove_SmartRemove = property(getAutoRemove_SmartRemove, setAutoRemove_SmartRemove)
	AutoRemove_NamedSnapshots = property(getAutoRemove_NamedSnapshots, setAutoRemove_NamedSnapshots)
	Schedule= property(getSchedule, setSchedule)
	Options_CompressFiles = property(getOptions_CompressFiles, setOptions_CompressFiles)
	Options_EncryptFilesEnabled = property(getOptions_EncryptFilesEnabled, setOptions_EncryptFilesEnabled)
	Options_EncryptionKey = property(getOptions_EncryptionKey, setOptions_EncryptionKey)
	Options_EnableNotifications = property(getOptions_EnableNotifications, setOptions_EnableNotifications)
	Options_ExcludeHidden = property(getOptions_ExcludeHidden, setOptions_ExcludeHidden)
	Options_FullBackupFrequencyEnabled = property(getOptions_FullBackupFrequencyEnabled, setOptions_FullBackupFrequencyEnabled)
	Options_FullBackupFrequency= property(getOptions_FullBackupFrequency, setOptions_FullBackupFrequency)
	Advanced_DefaultRestoreEnabled = property(getAdvanced_DefaultRestoreEnabled, setAdvanced_DefaultRestoreEnabled)
	Advanced_DefaultRestoreLocation = property(getAdvanced_DefaultRestoreLocation, setAdvanced_DefaultRestoreLocation)
	FolderList = property(getFolderList)
	ExcludeList = property(getExcludeList)
